/********************************************************************************
 * File: EntranceCommands.h
 * Definition: Contains the definition of the commands that will be used by players
 * 	at the entrance screen of the MUD to login to an account or player or to create
 *  a new account.
 *
 * Contents:
 * 		LoginCommand
 * 		CreateCommand
 * 		HelpCommand
 * 		EntranceInterpreter
 *
 * Copyright:
 *		Copyright (C) 2009-2011 Daniel Bingham (http://www.theroadgoeson.com)
 *
 * License:
 *
 * This software is licensed under the MIT Open Source License which reads as
 * follows:
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in the
 * Software without restriction, including without limitation the rights to use, copy,
 * modify, merge, publish, distribute, sublicense, and/or sell copies of the Software,
 * and to permit persons to whom the Software is furnished to do so, subject to the
 * following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies
 * or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
 * USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * For more information see here: http://www.opensource.org/licenses/mit-license.php
 *
 */

#ifndef ENTRACECOMMANDS_H_
#define ENTRANCECOMMANDS_H_

#include "ModelFactory.h"
#include "Interpreter.h"
#include "CommandInterpreter.h"

class Player;
class PlayerList;

class LoginCommand: public Command {
	public:
		LoginCommand(PlayerList *pl);
		~LoginCommand();

		void execute(Player *player, String arguments);
};

class CreateCommand: public Command {
	public:
		CreateCommand(PlayerList *pl);
		~CreateCommand();

		void execute(Player *player, String arguments);
};

class HelpCommand: public Command {
	public:
		HelpCommand(PlayerList *pl);
		~HelpCommand();

		void execute(Player *player, String arguments);
};

class EntranceInterpreter: public CommandInterpreter {
	public:
		EntranceInterpreter(PlayerList *pl);
		~EntranceInterpreter();

	protected:
		void initializeCommandList();
};

#endif /* ENTRANCECOMMANDS_H_ */
